(function () {

var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive("zhInfinityScroll", ["templateManager", "Query", "ZhUtil", "$compile", zhInfinityScrollDirective]);

function zhInfinityScrollDirective(templateManager, Query, ZhUtil, $compile) {
	return {
		scope: {
			zhInfinityScroll: "=",
			widget: "="
		},
		link: function(scope, element, attrs) {
			var widget = scope.widget;
			var datasource = scope.widget.dataSource;
			var itemsPerPage = widget.itemsPerPage || datasource.itemsPerPage;
			element.infinityOldValue = "";
			element.loadedAllData = false;
			element.previousScroll = 0;

			if (Util.isMobile()) {
				element.nonbounce();
			}

			var scrollMethod = Util.buildDebounceMethod(function(e, force) {
				// detect if scrolling vertically
				var verticalScroll = (element.scrollTop() - element.previousScroll) !== 0;
				element.previousScroll = element.scrollTop();

				if (element.infinityOldValue !== scope.zhInfinityScroll) {
					element.infinityOldValue = scope.zhInfinityScroll;
					element.loadedAllData = false;
				}

				var scrolling = (e !== undefined);
				if (scrolling || force) {
					var dataEl = element.is('.body-content') ? element : element.find("table"); //new_grid
					if (!dataEl.length) {
						dataEl = element.find("ul");
					}

					var maxCurrent = datasource.getCurrentPage() * itemsPerPage;

					if (!force) {
						if (!verticalScroll)
							return;
						else if (!widget.hasPagination)
							return;
						else if (dataEl.height() <= element.height() - 50)
							return;
						else if ((e.target.scrollTop + e.target.offsetHeight) < (e.target.scrollHeight - 10))
							return;
						else if (datasource.data.length < maxCurrent)
							return;
						else if (element.loadedAllData)
							return;
					}

					// remove "load more"
					element.find('.infinity-load-data').remove();
					delete scope.loadMore;
					delete widget.loadMore;
				}

				var filters = widget.getDataSourceFilter();
				if (Util.objectHasAnyValue(scope.zhInfinityScroll)) {
					filters = filters.concat(ZhUtil.getLikeAllFilter(widget, scope.zhInfinityScroll));
				}

				datasource.load(filters, datasource.getNextPage(), itemsPerPage, undefined, undefined, undefined, undefined, undefined, widget).then(function(result) {
					var data = null;
					if (result && result.dataset) {
						Object.keys(result.dataset).forEach(function (datasetName) {
							data = result.dataset[datasetName];
						});
					} else {
						data = result || [];
					}

					element.loadedAllData = data.length < itemsPerPage;

					if (element[0].scrollHeight > element.innerHeight()) {
						element.find('.infinity-load-data').remove();
						delete scope.loadMore;
						delete widget.loadMore;
					}

					if (data.length) {
						templateManager.updateTemplate();
					} else {
						datasource.getPrevPage();
					}

					widget.updateCheckedRows();
				});

				widget.__infinityScrolled = true;
			}, 500);
			element.on('scroll.infinity', scrollMethod);

			widget.infinityScroll = function() {
				widget.__infinityScrolled = true;
				element.find('.infinity-load-data').remove();
				delete scope.loadMore;
				delete widget.loadMore;
			};

			widget.resetInfinityScroll = function() {
				widget.__infinityScrolled = false;
			};

			scope.$on('$destroy', function() {
				delete widget.infinityScroll;
			});

			scope.$watch(function() {
				return widget.groupedData ? widget.groupedData : widget.dataSource.data;
			}, function(newValue) {
				if (!newValue) { return; }

				if (element.infinityOldValue !== scope.zhInfinityScroll || datasource.getCurrentPage() === 1) {
					element.infinityOldValue = scope.zhInfinityScroll;
					element.loadedAllData = false;
				}

				if (newValue.length >= itemsPerPage && !element.loadedAllData && !widget.__infinityScrolled && widget.hasPagination ) {
					if (element.find('.infinity-load-data').length === 0) {
						var loadMoreControl = $compile('<div class="infinity-load-data" ng-hide="widget._hideLoadMore" data-ng-click="loadMore();">{{"Load more"|i18n:words:lang}}</div>')(scope);
						scope.loadMore = widget.loadMore = function() {
							scrollMethod();
						};
						loadMoreControl.appendTo(element);
					}
				} else {
					element.find('.infinity-load-data').remove();
					delete scope.loadMore;
					delete widget.loadMore;
				}
			});
		}
	};
}

})();